'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _markdownTable = require('markdown-table');

var _markdownTable2 = _interopRequireDefault(_markdownTable);

var _helpers = require('./helpers');

var _types = require('./types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var TABLE_HEADERS = ['Name', 'Type', 'Default', 'Required', 'Description'];

/**
 * Loop through all the top level props
 * @param props
 * @param options
 * @returns {string}
 */
function addProps(props, options) {
  if (!props) return '## No props';
  var keys = Object.keys(props).filter(function (key) {
    return (0, _helpers.filterProps)(key, props[key], options);
  });
  var filteredProps = keys.reduce(function (last, key) {
    return _extends({}, last, _defineProperty({}, key, props[key]));
  }, {});

  var output = '\n## Props\n';

  var items = [TABLE_HEADERS].concat(_toConsumableArray(keys.map(function (key) {
    var prop = filteredProps[key];
    var row = [(0, _helpers.getKey)(key, (0, _helpers.getType)(prop)), (0, _helpers.getTypeName)((0, _helpers.getType)(prop)), (0, _helpers.getDefaultValue)(prop), prop.required, prop.description];
    return row.map(function (rowValue) {
      if (typeof rowValue === 'string') {
        return rowValue.split('\n').join('<br>');
      }
      return rowValue;
    });
  })));

  output += (0, _markdownTable2.default)(items) + '\n';

  // Add subtypes
  var subTypes = (0, _types.describeSubTypes)(filteredProps);
  if (subTypes.length) {
    output += '\n## Complex Props\n';
    output += subTypes;
  }

  return output;
}

function normalizeOptions(options) {
  return {
    excludeKey: toRegExp(options.excludeKeys || options.excludeKey),
    excludeType: toRegExp(options.excludeTypes || options.excludeType),
    excludeDescription: toRegExp(options.excludeDescription || options.excludeDescription || '@internal')
  };
}

function toRegExp(input) {
  if (!input) return null;
  if (Array.isArray(input)) return new RegExp('(' + input.join('|') + ')', 'i');

  return new RegExp(input, 'i');
}

/**
 *
 * @param api {object} The react-docgen output
 * @param name {string} Name/title to append on the page
 * @param options {Object}
 * @param options.excludeKeys {string|Array|RegExp} Prop keys to exclude from the docs. "children" will prevent children from showing up.
 * @param options.excludeTypes {string|Array|RegExp} Prop types to exclude from the docs. "Node" will prevent Node types from showing up.
 * @param options.excludeDescription {string|Array|RegExp} If description includes matching word, the item will not be included. Default is "@internal".
 *
 * @returns {string} Markdown document
 */
function docsToMarkdown(api) {
  var name = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var output = '';
  if (name) output += '# ' + name + '\n';
  if (api.description) output += '\n' + api.description + '\n\n';
  output += addProps(api.props, normalizeOptions(options));

  return output;
}

module.exports = docsToMarkdown;

exports.default = docsToMarkdown;